<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="html" indent="yes" />
	<xsl:param name="CAM_Template"/>
	<xsl:param name="XML_File"/>	
	<xsl:variable name="version">2.02</xsl:variable>
	<!-- Modified December, 2009 for CAMV ERRORCODE, CAMINFO, CAMWARN handling -->
	<!-- Change September, 2011 - remove CAMINFO - and fix multi-report line html output bug -->
	<!-- Change January 2012 - multi-error handling improved -->
	<!-- Change January 2012 - switch to XSLT v2.0 prolog and processor -->
	<xsl:template match="/">
		<html>
			<head>
				<meta content="text/html; charset=UTF-8" http-equiv="Content-Type"/>
				<title>CAM Exchange Validation Results Report</title>
			</head>
			
			<body>

				<p><img src="cam.jpg" alt="Logo"/></p>

				<p><br/></p>
				
				<table border="0" width="100%">
					<tr>
						<td>
							<b>Validation Results Report</b>
						</td>
						<td>
							<xsl:value-of select="$CAM_Template"/>
						</td>
						<td>Version: <xsl:value-of select="$version"/></td>
					</tr>
				</table>

				<p><br/></p>
				<xsl:variable name="countErrors">
					<xsl:for-each select="//node()/@*/name()">
						<xsl:if test="contains(.,'CAMERROR') or(contains(.,'CAMWARN'))"><error/></xsl:if>
					</xsl:for-each>
				</xsl:variable>
				
				<xsl:choose>
				<xsl:when test="count($countErrors//error) &gt; 0">
				<table border="0" width="100%" cellpadding="10">
					<tr  style="background-color:yellow">
						<th style="width:10%;height:35;text-align:left">
							<strong style="font:arial">Type</strong>
						</th>
						<th style="width:35%;height:35;text-align:left">
							<strong style="font:arial">Validation Detail</strong>
						</th>
						<th style="width:55%;height:35;text-align:left">
							<strong style="font:arial">Location</strong>
						</th>
						
					</tr>

					<!-- Now write the table rows out just here -->
					<xsl:apply-templates/>
					<!-- Now write the table last row at end -->
					<tr style="background-color:yellow">
						<th style="width:10%;height:35"></th>
						<th style="width:35%;height:35"></th>
						<th style="width:55%;height:35"></th>
					</tr>
				</table>
				</xsl:when>
				<xsl:otherwise>
					<table border="0" width="100%">
						<tr>
							<td>
								<b>Validation reported no problems</b>
							</td>
							<td>
							
							</td>
						</tr>
					</table>
				</xsl:otherwise>
				</xsl:choose>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="//*">
		<xsl:variable name="errors">
			<xsl:for-each select="@*">
				<xsl:variable name="chkName" select="name()"/>
				<xsl:if test="contains($chkName,'CAMERROR')"><error name="{$chkName}"><xsl:value-of select="."/></error></xsl:if>
				<xsl:if test="contains($chkName,'CAMWARN')"><error name="{$chkName}">WARNING: <xsl:value-of select="."/></error></xsl:if>
				<!-- xsl:if test="contains($chkName,'CAMINFO')"><error name="{$chkName}">INFO: <xsl:value-of select="."/></error></xsl:if -->
			</xsl:for-each>
		</xsl:variable>
		<xsl:if test="count($errors//error) &gt; 0">
		<xsl:variable name="thisPath"><xsl:call-template name="path"/></xsl:variable>
		
		<xsl:variable name="errorDetails">
			<xsl:for-each select="@*">
				<xsl:variable name="chkName" select="name()"/>
				<xsl:if test="contains($chkName,'CAMERROR') and(not(contains($chkName,'CAMERRORCODE')))"><error name="{$chkName}"><xsl:value-of select="."/></error></xsl:if>
				<xsl:if test="contains($chkName,'CAMWARN')"><error name="{$chkName}">WARNING: <xsl:value-of select="."/></error></xsl:if>
				<!-- xsl:if test="contains($chkName,'CAMINFO')"><error name="{$chkName}">INFO: <xsl:value-of select="."/></error></xsl:if -->
			</xsl:for-each>
		</xsl:variable>
		<xsl:for-each select="$errors//error">
			<xsl:variable name="this" select="position()"/>
			<xsl:variable name="fullPath">
				<xsl:value-of select="$thisPath"/>
				<!-- xsl:choose>
					<xsl:when test="@name='CAMERRORCODE'"><xsl:value-of select="$thisPath"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="concat($thisPath,'/@',substring-after(@name,'CAMERRORCODE'))"/></xsl:otherwise>
				</xsl:choose -->
			</xsl:variable>
			<xsl:choose>
				<xsl:when test="string-length(substring-after(.,';')) =0 ">
					<xsl:call-template name="printError">
						<xsl:with-param name="code" select="."/>
						<xsl:with-param name="error" select="$errorDetails/error[$this]"/>
						<xsl:with-param name="path" select="$fullPath"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="splitError">
						<xsl:with-param name="code" select="."/>
						<xsl:with-param name="error" select="$errorDetails/error[$this]"/>
						<xsl:with-param name="path" select="$fullPath"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		</xsl:if>
		<xsl:apply-templates select="child::*"/>
	</xsl:template>
	
	<xsl:template name="splitError">
		<xsl:param name="code"/>
		<xsl:param name="error"/>
		<xsl:param name="path"/>
		<!-- xsl:message>[<xsl:value-of select="$code"/>]</xsl:message -->
		<xsl:for-each select="tokenize($code,';')">
			<xsl:if test="string-length(.) &gt; 0">
			<xsl:variable name="tempPath">
				<xsl:choose>
					<xsl:when test="contains(.,'Attribute:')"><xsl:value-of select="concat($path,'/@',substring-after(substring-after(.,'Attribute:'),'/'))"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="$path"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:call-template name="printError">
				<xsl:with-param name="code" select="."/>
				<xsl:with-param name="error" select="."/>
				<xsl:with-param name="path" select="$tempPath"/>
			</xsl:call-template>
		 </xsl:if>
		</xsl:for-each>  
	</xsl:template>

	<xsl:template match="*|@*|text()|node()">
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template name="path">
		<xsl:variable name="name" select="name()"/>
		<xsl:for-each select="parent::*">
			<xsl:call-template name="path"/>
		</xsl:for-each>
		<xsl:value-of select="concat('/',name(),'[',count(preceding-sibling::*[name()=$name])+1,']')"/>
	</xsl:template>

	<xsl:template name="printError">
		<xsl:param name="code"/>
		<xsl:param name="error"/>
		<xsl:param name="path"/>
		<xsl:variable name="ErrorFormat">
			<xsl:variable name="tempErr" select="substring-before($error,':')"/>
			<xsl:choose>
				<xsl:when test="string-length($tempErr) &gt; 8">ERROR</xsl:when>
				<xsl:otherwise><xsl:value-of select="substring-before($error,':')"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
		<xsl:when test="$code=''"/>
		<xsl:when test="contains($error,'|')">
			<tr>
			<td style="width:10%;font-size:12">
					<xsl:value-of select="$ErrorFormat"/>:
			</td>
			<td style="width:35%;font-size:12">
					<xsl:value-of select="substring-after($error,'|')"/>
			</td>
			<td style="width:55%;font-size:12">
				<xsl:choose>
					<xsl:when test="contains($path,'|')"><xsl:value-of select="substring-before($path,'|')"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="$path"/></xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
		</xsl:when>
		<xsl:otherwise>
		<tr>
			<td style="width:10%;font-size:12">
					<xsl:value-of select="$ErrorFormat"/>:
			</td>
			<td style="width:35%;font-size:12">
					<xsl:value-of select="substring-after($error,':')"/>
			</td>
			<td style="width:55%;font-size:12">
					<xsl:value-of select="$path"/>
			</td>
		</tr>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
</xsl:stylesheet>
