<?xml version='1.0'?>
<xsl:stylesheet version="2.0"  xmlns:camed="http://jcam.org.uk/editor" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:as="http://www.oasis-open.org/committees/cam">
<xsl:output method="xml" indent="yes"/>
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: Martin Roberts, June, 2008 -->

    <!-- 

Purpose : Take a CAM template in CXF format and convert it to original Cam Format
Original : 04/22/2008 
Initials : mmer 
CAM ver : 1.1 
 
Updated - Inits - Change 
Initial Draft	mmer	June 16 2008
Remove duplicate rules logic added  drrw   09/14/08
Handle annotations drrw 10/02/08
Fix XPath issue with setChoice() drrw 03/04/13
Handle parameters in Header drrw 03/04/13

-->
<xsl:strip-space elements="*"/>

<xsl:variable name="version">1.03</xsl:variable>

<xsl:template match="/">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="as:CAM">
	<xsl:copy  >
		<xsl:copy-of select="@*"/>
		<xsl:apply-templates select="//as:namespace" mode="declare"/>
		<xsl:apply-templates select="as:Header"/>
		<xsl:apply-templates select="as:AssemblyStructure" mode="structure"/>
		<xsl:apply-templates select="//as:Structure" mode="rules"/>
		<xsl:apply-templates select="//as:Extension"/>
		<xsl:apply-templates select="as:include"/>
	</xsl:copy>
	
</xsl:template>

<xsl:template match="as:namespace" mode="declare">
	<xsl:namespace name="{@prefix}" select="normalize-space(.)"/>
</xsl:template>

<xsl:template match="@*|text()|*">
	
</xsl:template>

<xsl:template match="as:Namespaces">
</xsl:template>

<xsl:template match="as:Header">
	<xsl:comment> 
		Generated on <xsl:value-of  select="current-date()"/> from CXF by cxf2cam version <xsl:value-of select="$version"/><xsl:value-of select="'&#xD;'"/> 
	</xsl:comment>
	<xsl:copy>
		<xsl:apply-templates/>
		<xsl:copy-of select="/as:CAM/as:Parameters"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="as:Header/as:*[not(contains(name(),'Namespace'))]">
		<xsl:copy-of select="."/>
</xsl:template>

<xsl:template match="as:AssemblyStructure" mode="structure">
	<xsl:copy>
		<xsl:apply-templates select="as:Structure" mode="structure"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="as:Structure" mode="structure">
	<xsl:copy>
		<xsl:copy-of select="@*"/>
		<xsl:apply-templates mode="structure"/>
	</xsl:copy>	
</xsl:template>

<xsl:template match="as:Element" mode="structure">
	<xsl:element name="{@name}" namespace="{@namespace}">
		<xsl:apply-templates select="as:Attribute" mode="structure"/>
		<xsl:choose>
		<xsl:when test="@exampleValue">
			<xsl:value-of select="@exampleValue"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates select="as:Element" mode="structure"/>
		</xsl:otherwise>
		</xsl:choose>
	</xsl:element>
</xsl:template>

<xsl:template match="as:Attribute" mode="structure">
	<xsl:attribute name="{@name}" namespace="{@namespace}">
			<xsl:value-of select="@exampleValue"/>
	</xsl:attribute>
</xsl:template>

<xsl:template match="text()" mode="structure"/>

<xsl:template match="as:Structure" mode="rules">

  <as:BusinessUseContext>
    <as:Rules>
      <as:default>
        <as:context>
        	<xsl:variable name="defaultRules">
			<xsl:apply-templates select="//as:constraint[not(exists(@context)) and @xpath]" mode="rules"/>
        	</xsl:variable>
        	<xsl:for-each select="$defaultRules/as:constraint">
        		<xsl:variable name="this" select="@action"/>
        		<xsl:variable name="count" select="count($defaultRules/as:constraint[@action=$this])"/>
        		<xsl:choose>
        			<xsl:when test="$count &gt; 1">
        				<xsl:variable name="chk" select="./following::*" />
        				<xsl:choose>
        					<xsl:when test="count($chk[@action=$this]) = 0"><xsl:copy-of select="."/></xsl:when>
        					<xsl:otherwise><!-- xsl:message>duplicate: <xsl:value-of select="$this"/></xsl:message --></xsl:otherwise>
        				</xsl:choose>					
        			</xsl:when>
        			<xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
        		</xsl:choose>	
        	</xsl:for-each>
        </as:context>
      </as:default>
	<xsl:apply-templates select="//as:constraint[exists(@context) and @xpath and @condition=@context]" mode="context"/>
    </as:Rules>
  </as:BusinessUseContext>
  <xsl:if test="count(//camed:annotation) &gt; 0">
  	<as:Extension xmlns:as="http://www.oasis-open.org/committees/cam" name="uk.org.jcam.camed.extensions.StructureAnnotations">
  		<xsl:apply-templates select="//camed:annotation" mode="notes"/>
  	</as:Extension>	
  </xsl:if>
</xsl:template>

<xsl:template match="as:constraint" mode="rules">
	<as:constraint>
		<xsl:if test="@condition and not(@context)">
			<xsl:attribute name="condition" select="@condition"/>
		</xsl:if>
		<xsl:variable name="start" select="substring-before(@action,'(')"/>
		<xsl:variable name="end" select="if (string-length(substring-after(@action,'(')) > 1 )then concat(',',substring-after(@action,'(')) else ')'"/>
		<xsl:attribute name="action" select="if($start = 'setChoice') then concat($start,'(',@xpath,')') else concat($start,'(',@xpath,$end)"/>
		<xsl:copy-of select="as:annotation"/>
	</as:constraint>
</xsl:template>
<xsl:template match="as:constraint" mode="context">
	<as:context>
		<xsl:if test="@context">
			<xsl:attribute name="condition" select="@context"/>
		</xsl:if>
		<xsl:variable name="context" select="@context"/>
		<xsl:apply-templates select="//as:constraint[contains(@condition,$context) and @context!=$context]" mode="context"/>
		<xsl:apply-templates select="//as:constraint[exists(@context) and @context=$context]" mode="rules"/>
	</as:context>
</xsl:template>

<xsl:template match="as:Extension[@name='uk.org.jcam.camed.extensions.StructureAnnotations']">
	<xsl:copy-of select="."/>
</xsl:template>
<xsl:template match="as:Extension[@name='uk.org.jcam.camed.extensions.LookupLists']">
	<xsl:copy-of select="." />
</xsl:template>

<xsl:template match="as:include">
	<xsl:copy-of select="."/>
</xsl:template>

<xsl:template match="camed:annotation" mode="notes">
		<xsl:copy-of select="."/>
</xsl:template>

</xsl:stylesheet>
